@@ -61,12 +61,6 @@
 #define COLOR_WARNING           "#FFE500"
 #define COLOR_URGENT            "#FF4F00"
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
-#endif
-
 // }}}
 
 // struct {{{
@@ -98,8 +92,6 @@ typedef struct
     GtkWidget          *cb_hide_button;
 } FsGuard;
 
-static GtkTooltips *tooltips = NULL;
-
 // }}}
 
 // all functions {{{
@@ -144,13 +136,10 @@ fsguard_set_icon (FsGuard *fsguard, gint id)
         return;
 
     fsguard->icon_id = id;
-	size = xfce_panel_plugin_get_size (fsguard->plugin);
-#ifdef HAS_PANEL_49
+    size = xfce_panel_plugin_get_size (fsguard->plugin);
     size /= xfce_panel_plugin_get_nrows (fsguard->plugin);
-#endif
 
-    size -= 2 + 2 * MAX (fsguard->btn_panel->style->xthickness,
-                         fsguard->btn_panel->style->ythickness);
+    size -= 2;
 
     icon_theme = gtk_icon_theme_get_default ();
     if (id == ICON_URGENT) {
@@ -162,7 +151,7 @@ fsguard_set_icon (FsGuard *fsguard, gint id)
     }
 
     if (G_UNLIKELY (NULL == pixbuf)) {
-        pixbuf = gtk_icon_theme_load_icon (icon_theme, GTK_STOCK_HARDDISK, size, 0, NULL);
+        pixbuf = gtk_icon_theme_load_icon (icon_theme, "gtk-harddisk", size, 0, NULL);
     }
 
     if (G_UNLIKELY (NULL == pixbuf)) {
@@ -190,32 +179,52 @@ fsguard_refresh_icon (FsGuard *fsguard)
 static void
 fsguard_refresh_monitor (FsGuard *fsguard)
 {
-    GdkColor            color;
+    GdkRGBA             color;
 
     switch (fsguard->icon_id) {
       default:
       case ICON_NORMAL:
-        gdk_color_parse (COLOR_NORMAL, &color);
+        gdk_rgba_parse (&color, COLOR_NORMAL);
         break;
 
       case ICON_WARNING:
-        gdk_color_parse (COLOR_WARNING, &color);
+        gdk_rgba_parse (&color, COLOR_WARNING);
         break;
 
       case ICON_URGENT:
-        gdk_color_parse (COLOR_URGENT, &color);
+        gdk_rgba_parse (&color, COLOR_URGENT);
         break;
     }
 
-    gtk_widget_modify_bg (GTK_WIDGET (fsguard->progress_bar),
+#if GTK_CHECK_VERSION (3, 16, 0)
+    GtkCssProvider *css_provider;
+#if GTK_CHECK_VERSION (3, 20, 0)
+    gchar * css = g_strdup_printf("progressbar trough { min-width: 4px; min-height: 4px; } \
+                                   progressbar progress { min-width: 4px; min-height: 4px; \
+                                                          background-color: %s; background-image: none; }",
+#else
+    gchar * css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }",
+#endif
+                                  gdk_rgba_to_string(&color));
+    /* Setup Gtk style */
+    css_provider = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+    gtk_style_context_add_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (fsguard->progress_bar))),
+        GTK_STYLE_PROVIDER (css_provider),
+        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_free(css);
+#else
+    gtk_widget_override_background_color (GTK_WIDGET (fsguard->progress_bar),
                           GTK_STATE_PRELIGHT,
                           &color);
-    gtk_widget_modify_bg (GTK_WIDGET (fsguard->progress_bar),
+    gtk_widget_override_background_color (GTK_WIDGET (fsguard->progress_bar),
                           GTK_STATE_SELECTED,
                           &color);
-    gtk_widget_modify_base (GTK_WIDGET (fsguard->progress_bar),
+    gtk_widget_override_color (GTK_WIDGET (fsguard->progress_bar),
                             GTK_STATE_SELECTED,
                             &color);
+#endif
 }
 
 static inline gboolean
@@ -246,7 +255,7 @@ fsguard_open_mnt (GtkWidget *widget, FsGuard *fsguard)
       return;
 
     GtkWidget *dialog;
-    dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_NO_SEPARATOR, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+    dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                                      "Free Space Checker");
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               _("Unable to find an appropriate application to open the mount point"));
@@ -282,22 +291,23 @@ fsguard_check_fs (FsGuard *fsguard)
             icon_id = ICON_URGENT;
         }
     }
+        g_snprintf (msg, sizeof (msg),
+                    _("could not check mountpoint %s, please check your config"),
+                    fsguard->path);
 
     /* msg_total_size, msg_size */
     if (total > 1024) {
         g_snprintf (msg_total_size, sizeof(msg_total_size), _("%.2f GB"), total / 1024);
         g_snprintf (msg_size, sizeof (msg_size), _("%.2f GB"), freespace / 1024);
+    } else {
+        g_snprintf (msg_total_size, sizeof (msg_total_size), _("%.0f MB"), total);
+        g_snprintf (msg_size, sizeof (msg_size), _("%.0f MB"), freespace);
+    }
+    if (err != -1)
         g_snprintf (msg, sizeof (msg),
                     (*(fsguard->name) != '\0' && strcmp(fsguard->path, fsguard->name)) ?
                     _("%s/%s space left on %s (%s)") : _("%s/%s space left on %s"),
                     msg_size, msg_total_size, fsguard->path, fsguard->name);
-    } else {
-        g_snprintf (msg_total_size, sizeof (msg_total_size), _("%.0f MB"), total);
-        g_snprintf (msg_size, sizeof (msg_size), _("%.0f MB"), freespace);
-        g_snprintf (msg, sizeof (msg),
-                    _("could not check mountpoint %s, please check your config"),
-                    fsguard->path);
-    }
 
     if (fsguard->show_size) {
         gtk_label_set_text (GTK_LABEL(fsguard->lab_size),
@@ -309,7 +319,7 @@ fsguard_check_fs (FsGuard *fsguard)
         fsguard_refresh_monitor (fsguard);
     }
 
-    gtk_tooltips_set_tip (tooltips, fsguard->ebox, msg, NULL);
+    gtk_widget_set_tooltip_text(fsguard->ebox, msg);
     fsguard_set_icon (fsguard, icon_id);
 
     if (err != -1 && !fsguard->seen && icon_id == ICON_URGENT) {
@@ -397,35 +407,32 @@ static FsGuard *
 fsguard_new (XfcePanelPlugin *plugin)
 {
     FsGuard *fsguard = g_new0(FsGuard, 1);
-    GtkWidget *alignment;
 
     fsguard->plugin = plugin;
 
     fsguard_read_config (fsguard);
 
-    tooltips = gtk_tooltips_new ();
-
     fsguard->ebox = gtk_event_box_new();
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(fsguard->ebox), FALSE);
 
     GtkOrientation orientation = xfce_panel_plugin_get_orientation (plugin);
-    fsguard->box = xfce_hvbox_new (orientation, FALSE, 2);
+    fsguard->box = gtk_box_new (orientation, 2);
 
     fsguard->lab_name = gtk_label_new (NULL);
     fsguard->lab_size = gtk_label_new (NULL);
-    fsguard->lab_box = xfce_hvbox_new (GTK_ORIENTATION_VERTICAL, FALSE, 2);
+    fsguard->lab_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
 
-    alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+    gtk_widget_set_halign(fsguard->lab_box, GTK_ALIGN_CENTER);
+    gtk_widget_set_valign(fsguard->lab_box, GTK_ALIGN_CENTER);
 
     fsguard->btn_panel = xfce_create_panel_button ();
     fsguard->icon_panel = gtk_image_new ();
 
     fsguard->progress_bar = gtk_progress_bar_new ();
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(fsguard->progress_bar), 0.0);
-    gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
-                                      orientation == GTK_ORIENTATION_HORIZONTAL ?
-                                      GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
-    fsguard->pb_box = xfce_hvbox_new (orientation, FALSE, 0);
+    gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR(fsguard->progress_bar), (orientation == GTK_ORIENTATION_HORIZONTAL));
+    gtk_orientable_set_orientation (GTK_ORIENTABLE(fsguard->progress_bar), !orientation);
+    fsguard->pb_box = gtk_box_new (orientation, 0);
 
     g_signal_connect (G_OBJECT(fsguard->btn_panel),
                       "clicked",
@@ -437,8 +444,7 @@ fsguard_new (XfcePanelPlugin *plugin)
     gtk_container_add (GTK_CONTAINER(fsguard->btn_panel), fsguard->icon_panel);
     gtk_container_add (GTK_CONTAINER(fsguard->lab_box), fsguard->lab_name);
     gtk_container_add (GTK_CONTAINER(fsguard->lab_box), fsguard->lab_size);
-    gtk_container_add (GTK_CONTAINER(fsguard->box), alignment);
-    gtk_container_add (GTK_CONTAINER(alignment), fsguard->lab_box);
+    gtk_container_add (GTK_CONTAINER(fsguard->box), fsguard->lab_box);
     gtk_container_add (GTK_CONTAINER(fsguard->box), fsguard->pb_box);
     gtk_container_add (GTK_CONTAINER(fsguard->pb_box), fsguard->progress_bar);
 
@@ -476,9 +482,7 @@ static gboolean
 fsguard_set_size (XfcePanelPlugin *plugin, int size, FsGuard *fsguard)
 {
     int border_width = (size > 26 ? 2 : 1);
-#ifdef HAS_PANEL_49
     size /= xfce_panel_plugin_get_nrows (plugin);
-#endif
     DBG ("Set size to `%d'", size);
 
     gtk_container_set_border_width (GTK_CONTAINER (fsguard->pb_box), border_width);
@@ -500,7 +504,6 @@ fsguard_set_size (XfcePanelPlugin *plugin, int size, FsGuard *fsguard)
     return TRUE;
 }
 
-#ifdef HAS_PANEL_49
 static void
 fsguard_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode, FsGuard *fsguard)
 {
@@ -516,16 +519,15 @@ fsguard_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode, FsGuard *fs
     DBG ("Set mode to `%s'", mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL ?
          "Horizontal" : (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL ? "Vertical" : "Deskbar"));
 
-    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->box), panel_orientation);
-    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->pb_box), panel_orientation);
-    gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
-                                      panel_orientation == GTK_ORIENTATION_HORIZONTAL ?
-                                      GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
+    gtk_orientable_set_orientation (GTK_ORIENTABLE (fsguard->box), panel_orientation);
+    gtk_orientable_set_orientation (GTK_ORIENTABLE (fsguard->pb_box), panel_orientation);
+    gtk_orientable_set_orientation (GTK_ORIENTABLE (fsguard->progress_bar), !panel_orientation);
+    gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR(fsguard->progress_bar), (panel_orientation == GTK_ORIENTATION_HORIZONTAL));
     gtk_label_set_angle (GTK_LABEL(fsguard->lab_name),
                          orientation == GTK_ORIENTATION_VERTICAL ? -90 : 0);
     gtk_label_set_angle (GTK_LABEL(fsguard->lab_size),
                          orientation == GTK_ORIENTATION_VERTICAL ? -90 : 0);
-    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->lab_box),
+    gtk_orientable_set_orientation (GTK_ORIENTABLE (fsguard->lab_box),
                                 orientation == GTK_ORIENTATION_VERTICAL ?
                                 GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
     gtk_box_reorder_child (GTK_BOX (fsguard->lab_box),
@@ -533,22 +535,6 @@ fsguard_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode, FsGuard *fs
     fsguard_set_size (plugin, xfce_panel_plugin_get_size (plugin), fsguard);
 }
 
-#else
-static void
-fsguard_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation, FsGuard *fsguard)
-{
-    DBG ("Set orientation to `%s'", orientation == GTK_ORIENTATION_HORIZONTAL ?
-                                    "Horizontal" : "Vertical");
-
-    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->box), orientation);
-    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->pb_box), orientation);
-    gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
-                                      orientation == GTK_ORIENTATION_HORIZONTAL ?
-                                      GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
-    fsguard_set_size (plugin, xfce_panel_plugin_get_size (plugin), fsguard);
-}
-#endif
-
 static void
 fsguard_entry1_changed (GtkWidget *widget, FsGuard *fsguard)
 {
@@ -634,58 +620,58 @@ fsguard_create_options (XfcePanelPlugin *plugin, FsGuard *fsguard)
     GtkWidget *dialog =
       xfce_titled_dialog_new_with_buttons (_("Free Space Checker"),
         GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
-        GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-        GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+        GTK_DIALOG_DESTROY_WITH_PARENT,
+        "gtk-close", GTK_RESPONSE_OK,
         NULL);
     gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce4-fsguard-plugin-warning");
     gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
     /* Configuration frame */
-    GtkWidget *table1 = gtk_table_new (2, 3, FALSE);
+    GtkWidget *table1 = gtk_grid_new ();
     GtkWidget *frame1 = xfce_gtk_frame_box_new_with_content (_("Configuration"), table1);
-    gtk_table_set_row_spacings (GTK_TABLE (table1), BORDER);
-    gtk_table_set_col_spacings (GTK_TABLE (table1), BORDER);
+    gtk_grid_set_row_spacing (GTK_GRID (table1), BORDER);
+    gtk_grid_set_column_spacing (GTK_GRID (table1), BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (frame1), BORDER);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame1,
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dialog))), frame1,
                         TRUE, TRUE, 0);
 
     GtkWidget *label1 = gtk_label_new (_("Mount point"));
-    gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
+    gtk_widget_set_valign(label1, GTK_ALIGN_CENTER);
     GtkWidget *entry1 = gtk_entry_new ();
     gtk_entry_set_max_length (GTK_ENTRY (entry1), 32);
     gtk_entry_set_text (GTK_ENTRY (entry1), fsguard->path);
 
     GtkWidget *label3 = gtk_label_new (_("Warning limit (%)"));
-    gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);
+    gtk_widget_set_valign(label3, GTK_ALIGN_CENTER);
     GtkWidget *spin1 = gtk_spin_button_new_with_range (0, 100, 1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin1), fsguard->limit_warning);
 
     GtkWidget *label4 = gtk_label_new (_("Urgent limit (%)"));
-    gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);
+    gtk_widget_set_valign(label4, GTK_ALIGN_CENTER);
     GtkWidget *spin2 = gtk_spin_button_new_with_range (0, 100, 1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin2), fsguard->limit_urgent);
 
-    gtk_table_attach_defaults (GTK_TABLE (table1), label1,
-                               0, 1, 0, 1);
-    gtk_table_attach_defaults (GTK_TABLE (table1), entry1,
-                               1, 2, 0, 1);
-    gtk_table_attach_defaults (GTK_TABLE (table1), label3,
-                               0, 1, 1, 2);
-    gtk_table_attach_defaults (GTK_TABLE (table1), spin1,
-                               1, 2, 1, 2);
-    gtk_table_attach_defaults (GTK_TABLE (table1), label4,
-                               0, 1, 2, 3);
-    gtk_table_attach_defaults (GTK_TABLE (table1), spin2,
-                               1, 2, 2, 3);
+    gtk_grid_attach (GTK_GRID (table1), label1,
+                               0, 0, 1, 1);
+    gtk_grid_attach (GTK_GRID (table1), entry1,
+                               1, 0, 1, 1);
+    gtk_grid_attach (GTK_GRID (table1), label3,
+                               0, 1, 1, 1);
+    gtk_grid_attach (GTK_GRID (table1), spin1,
+                               1, 1, 1, 1);
+    gtk_grid_attach (GTK_GRID (table1), label4,
+                               0, 2, 1, 1);
+    gtk_grid_attach (GTK_GRID (table1), spin2,
+                               1, 2, 1, 1);
 
     /* Display frame */
-    GtkWidget *table2 = gtk_table_new (2, 4, FALSE);
+    GtkWidget *table2 = gtk_grid_new ();
     GtkWidget *frame2 = xfce_gtk_frame_box_new_with_content (_("User Interface"), table2);
-    gtk_table_set_row_spacings (GTK_TABLE (table2), BORDER);
-    gtk_table_set_col_spacings (GTK_TABLE (table2), BORDER);
+    gtk_grid_set_row_spacing (GTK_GRID (table2), BORDER);
+    gtk_grid_set_column_spacing (GTK_GRID (table2), BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (frame2), BORDER);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame2,
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dialog))), frame2,
                         TRUE, TRUE, 0);
 
     GtkWidget *check1 = gtk_check_button_new_with_label (_("Name"));
@@ -708,16 +694,16 @@ fsguard_create_options (XfcePanelPlugin *plugin, FsGuard *fsguard)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fsguard->cb_hide_button),
                                   !fsguard->hide_button);
 
-    gtk_table_attach_defaults (GTK_TABLE (table2), check1,
-                               0, 1, 0, 1);
-    gtk_table_attach_defaults (GTK_TABLE (table2), entry3,
-                               1, 2, 0, 1);
-    gtk_table_attach_defaults (GTK_TABLE (table2), check2,
-                               0, 2, 1, 2);
-    gtk_table_attach_defaults (GTK_TABLE (table2), check3,
-                               0, 2, 2, 3);
-    gtk_table_attach_defaults (GTK_TABLE (table2), fsguard->cb_hide_button,
-                               0, 2, 3, 4);
+    gtk_grid_attach (GTK_GRID (table2), check1,
+                               0, 0, 1, 1);
+    gtk_grid_attach (GTK_GRID (table2), entry3,
+                               1, 0, 1, 1);
+    gtk_grid_attach (GTK_GRID (table2), check2,
+                               0, 1, 2, 1);
+    gtk_grid_attach (GTK_GRID (table2), check3,
+                               0, 2, 2, 1);
+    gtk_grid_attach (GTK_GRID (table2), fsguard->cb_hide_button,
+                               0, 3, 2, 1);
 
     g_signal_connect (entry1,
                       "changed",
@@ -752,7 +738,7 @@ fsguard_create_options (XfcePanelPlugin *plugin, FsGuard *fsguard)
                       G_CALLBACK (fsguard_check4_changed),
                       fsguard);
 
-    gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
+    gtk_widget_show_all (GTK_WIDGET (dialog));
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
     xfce_panel_plugin_unblock_menu (fsguard->plugin);
@@ -787,18 +773,11 @@ fsguard_construct (XfcePanelPlugin *plugin)
                       "size-changed",
                       G_CALLBACK (fsguard_set_size),
                       fsguard);
-#ifdef HAS_PANEL_49
     g_signal_connect (plugin,
                       "mode-changed",
                       G_CALLBACK (fsguard_set_mode),
                       fsguard);
     xfce_panel_plugin_set_small (plugin, TRUE);
-#else
-    g_signal_connect (plugin,
-                      "orientation-changed",
-                      G_CALLBACK (fsguard_set_orientation),
-                      fsguard);
-#endif
     g_signal_connect (plugin,
                       "configure-plugin",
                       G_CALLBACK (fsguard_create_options),
